"{ Package: 'stx:libscm/mercurial' }"

SCMAbstractPackageModel subclass:#HGPackageModel
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Mercurial-StX'
!


!HGPackageModel class methodsFor:'instance creation'!

named:package
    ^HGPackageModelRegistry packageNamed: package

    "Modified: / 16-11-2012 / 19:53:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGPackageModel methodsFor:'accessing'!

manager
    ^ HGSourceCodeManager

    "Created: / 14-11-2012 / 01:02:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

repository
    self ensureWorkingCopy.
    ^wc repository

    "Created: / 15-11-2012 / 09:47:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

revision
    "Return a logical revision of the package, i.e., a revision
     on which the next commit will be based on"

    ^self definition hgLogicalRevision

    "Created: / 28-11-2012 / 09:41:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

upstreamRepository
    "Returns original package repository (not the temporary repository)"

    "Stupid naming here..."

    ^repository

    "Created: / 27-11-2012 / 23:22:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGPackageModel methodsFor:'accessing-classes'!

commitDialogClass
    "raise an error: must be redefined in concrete subclass(es)"

    ^ HGCommitDialog

    "Created: / 14-11-2012 / 22:30:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

commitTaskClass
    ^ HGCommitTask

    "Created: / 14-11-2012 / 00:36:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGPackageModel methodsFor:'private'!

commited
    "Sent by commit task once commited"

    | versionMethod |

    super commited.

    versionMethod := self definition class compiledMethodAt: HGSourceCodeManager nameOfVersionMethodInClasses.
    versionMethod isNil ifTrue:[
        self error:'Should not happen!!'.
    ].
    versionMethod annotateWith: 
        (HGRevisionAnnotation revision: wc changeset id)

    "Created: / 23-11-2012 / 22:52:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGPackageModel methodsFor:'utils'!

ensureWorkingCopy
    "raise an error: must be redefined in concrete subclass(es)"

    | rev tmpPath tmpRepo tmpWc |

    rev := self definition hgLogicalRevision.
    (wc notNil and:[wc path exists]) ifTrue:[ ^ self ].

    tmpPath := self manager temporaryWorkingCopyDirectory / repository uuid printString.
    tmpPath exists ifTrue:[
        tmpRepo := HGRepository on: tmpPath.
        tmpRepo pull.
    ] ifFalse:[
        tmpRepo := repository cloneTo: tmpPath update: false.
    ].
    tmpWc := tmpRepo workingCopy.
    tmpWc update: rev.

    self wc: tmpWc

    "Created: / 14-11-2012 / 00:16:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 28-11-2012 / 09:38:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ensureWorkingCopyAtRevision: rev
    "Make sure temporary working exists and is at
     given revision"

    self assert: rev revno notNil.

    self ensureWorkingCopy.
    rev notNil ifTrue:[
        wc update: rev revno printString. 
    ].

    "Created: / 28-11-2012 / 09:38:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 30-11-2012 / 23:22:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGPackageModel class methodsFor:'documentation'!

version_GIT
    "Never, ever change this method. Ask JV or CG why"
    ^thisContext method mclass theNonMetaclass instVarNamed: #revision
!

version_HG

    ^ '$Changeset: <not expanded> $'
!

version_SVN
    ^ 'Id::                                                                                                                        '
! !
